@@ -10,7 +10,7 @@ use warnings;
 
 use utf8;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Carp;
 
@@ -228,6 +228,19 @@ Pins are specified in an C<ARRAY> reference giving the names of pins (as
 returned by the L</list_gpios> method); read values are given in the returned
 C<HASH> reference which maps pin names to boolean logic levels.
 
+=head2 tris_gpios
+
+   $protocol->tris_gpios( \@pin_names )->get;
+
+Sets the named GPIO pins as high-impedence inputs ("tristate"). Any GPIO pins
+not named here are left as they are.
+
+This method is similar to L</read_gpios> except that it does not return the
+current pin values to the caller. Adapter implementations may implement this
+by simply calling L</read_gpios> or they may have a more efficient variant
+that does not have to transfer these extra readings back from the adapter
+hardware.
+
 =cut
 
 =head1 GPIO PROTOCOL
@@ -261,10 +274,11 @@ communication may happen at some rate slower than this.
 
    $bytes_in = $spi->readwrite( $bytes_out )->get
 
-Performs a complete SPI transaction; synchronously clocking the data given by
-the plain byte string I<$bytes_out> out of the MOSI pin of the adapter, while
-simultaneously capturing the data coming in to the MISO pin, which is
-eventually returned as the result of the returned future.
+Performs a complete SPI transaction; assert the SS pin, synchronously clock the
+data given by the plain byte string I<$bytes_out> out of the MOSI pin of the
+adapter while simultaneously capturing the data coming in to the MISO pin, then
+release the SS pin again. The bytes clocked in are eventually returned as the
+result of the returned future.
 
 =head2 write
 
@@ -276,6 +290,54 @@ This may or may not make a material difference to the actual communication
 with the adapter or device; it could be implemented simply by calling the
 C<readwrite> method and ignoring the return value.
 
+=head2 assert_ss
+
+=head2 release_ss
+
+   $spi->assert_ss->get
+
+   $spi->release_ss->get
+
+Lower-level access methods to directly assert or release the SS pin of the
+adapter. These would typically be used in conjunction with L</readwrite_no_ss>
+or L</write_no_ss>.
+
+=head2 readwrite_no_ss
+
+=head2 write_no_ss
+
+   $bytes_in = $spi->readwrite_no_ss( $bytes_out )->get
+
+   $spi->write_no_ss( $bytes )->get
+
+Lower-level access methods to directly perform a byte transfer across the
+MOSI/MISO pins of the adapter, without touching the SS pin. A complete SPI
+transaction can be performed in conjunction with the L</assert_ss> and
+L</release_ss> methods.
+
+   $spi->assert_ss
+      ->then( sub {
+         $spi->readwrite_no_ss( $bytes_out );
+      })
+      ->then( sub {
+         ( $bytes_in ) = @_;
+         $spi->release_ss
+      });
+
+These methods are provided for situations where it is not possible to know in
+advance all the bytes to be sent out in an SPI transaction; where the chip
+driver code must inspect some of the bytes incoming before it can determine
+what else needs to be sent, but when these must all be sent in one SS-asserted
+transaction.
+
+Because they perform multiple independent operations on the underlying
+adapter, these lower-level methods may be less efficient than using the single
+higher-level methods of L</readwrite> and L</write>. As such, they should only
+be used when the combined higher-level method cannot be used. However, as an
+implementation detail, a device adapter module could implement the higher-
+level methods by using the lower ones as described above, if that would be an
+efficient implementation.
+
 =cut
 
 =head1 I2C PROTOCOL
