@@ -9,12 +9,12 @@ use strict;
 use warnings;
 use base qw( Device::Chip::Adapter );
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Carp;
 
 use Future;
-use List::Util qw( first );
+use List::Util 1.33 qw( first any );
 use Test::Builder;
 
 =head1 NAME
@@ -57,6 +57,13 @@ sub new
    }, $class;
 }
 
+sub make_protocol_GPIO
+{
+   my $self = shift;
+   $self->{protocol} = "GPIO";
+   return Future->done( $self );
+}
+
 sub make_protocol_I2C
 {
    my $self = shift;
@@ -82,6 +89,10 @@ sub _stringify
    if( $v =~ m/^-?[0-9]+$/ ) {
       return sprintf "0x%X", $v;
    }
+   elsif( $v =~ m/^[\x20-\x7E]*$/ ) {
+      $v =~ s/([\\'])/\\$1/g;
+      return qq('$v');
+   }
    else {
       $v =~ s{(.)}{sprintf "\\x%02X", ord $1}gse;
       return qq("$v");
@@ -100,9 +111,16 @@ associated expectation method, whose name is prefixed C<expect_>. Each returns
 an expectation object, which has additional methods to control the behaviour of
 that invocation.
 
+ $exp = $adapter->expect_write_gpios( \%gpios )
+ $exp = $adapter->expect_read_gpios( \@gpios )
+ $exp = $adapter->expect_tris_gpios( \@gpios )
  $exp = $adapter->expect_write( $bytes )
  $exp = $adapter->expect_write_then_read( $bytes, $len )
- $exp = $apapter->expect_readwrite( $bytes_out )
+ $exp = $adapter->expect_readwrite( $bytes_out )
+ $exp = $adapter->expect_assert_ss
+ $exp = $adapter->expect_release_ss
+ $exp = $adapter->expect_readwrite_no_ss( $bytes_out )
+ $exp = $adapter->expect_write_no_ss( $bytes )
 
 The returned expectation object allows the test script to specify what such an
 invocation should return.
@@ -111,12 +129,35 @@ invocation should return.
 
 =cut
 
-my @METHODS = qw(
-   write write_then_read readwrite
+my %METHODS = (
+   write_gpios     => [ sub { my ( $v ) = @_; join ",", map { $v->{$_} ? $_ : "!$_" } sort keys %$v },
+                        [qw( GPIO SPI I2C )] ],
+   read_gpios      => [ sub { my ( $v ) = @_; join ",", @$v },
+                        [qw( GPIO SPI I2C )] ],
+   tris_gpios      => [ sub { my ( $v ) = @_; join ",", @$v },
+                        [qw( GPIO SPI I2C )] ],
+   write           => [ undef,
+                        [qw( SPI I2C )] ],
+   write_then_read => [ undef,
+                        [qw( I2C )] ],
+   readwrite       => [ undef,
+                        [qw( SPI )] ],
+   assert_ss       => [ undef,
+                        [qw( SPI )] ],
+   release_ss      => [ undef,
+                        [qw( SPI )] ],
+   write_no_ss     => [ undef,
+                        [qw( SPI )] ],
+   readwrite_no_ss => [ undef,
+                        [qw( SPI )] ],
 );
-foreach my $method ( @METHODS ) {
+foreach my $method ( keys %METHODS ) {
+   my ( $canonicalise, $allowed_protos ) = @{ $METHODS{$method} };
+
    my $expect = sub {
       my $self = shift;
+      @_ = $canonicalise->( @_ ) if $canonicalise;
+
       push @{ $self->{expect} }, my $e = bless [ $method, [ @_ ], 0, [] ],
          "Test::Device::Chip::Adapter::_Expectation";
 
@@ -125,8 +166,13 @@ foreach my $method ( @METHODS ) {
 
    my $actual = sub {
       my $self = shift;
+      @_ = $canonicalise->( @_ ) if $canonicalise;
+
       my @args = @_;
 
+      any { $_ eq $self->{protocol} } @$allowed_protos or
+         croak "Method ->$method not allowed in $self->{protocol} protocol";
+
       my $e = first { not $_->complete } @{ $self->{expect} };
 
       $e and $e->consume( $method, \@args ) or
@@ -231,7 +277,7 @@ sub returns
 
 =item *
 
-Handle GPIO methods
+Handle C<list_gpios> method
 
 =item *
 
